//*********************************************************************************************
// component.cpp
//*********************************************************************************************

#include "StdAfx.h"

#include "component.h"

//----------------------------------------------------------------------------------------------

// Functor for total ordering on components
bool less_Component::operator()(const CComponent* C1, const CComponent* C2) const
{
  return (*C1 < *C2);
}

//----------------------------------------------------------------------------------------------

// Construct isolated vertex
CComponent::CComponent(int v)
{
  _V.add(v);
}

//----------------------------------------------------------------------------------------------

// Construct component from single edge e
CComponent::CComponent(const CEdge& e)
{
  _V.add(e._v);
  _V.add(e._w);
  _E.add(e);
}

//----------------------------------------------------------------------------------------------

// Construct component from two disjoint subcomponents connected by edge f
CComponent::CComponent(const CComponent* C1, const CComponent* C2, const CEdge& f)
{
  _V.combine(C1->_V, C2->_V);
  _E.combine(C1->_E, C2->_E);
  assert(_E.add(f));
}

//----------------------------------------------------------------------------------------------

// Total order on components
// Precedence is given to components with fewer edges, hence isolated vertex 1 is least element.
// (U, E) < (V, F) :<=> |E| < |F|         if |E| != |F|
//                      U_1 < V_1         if both components are isolated vertices
//                      E < F under lex   otherwise       
bool CComponent::operator <(const CComponent& rhs) const
{
  if (_E.size() != rhs._E.size()) return (_E.size() < rhs._E.size());
  if (_E.size() == 0) return (isolated_vertex() < rhs.isolated_vertex());
  return (_E < rhs._E);
}

//----------------------------------------------------------------------------------------------

// Return isolated vertex V_1 (only to be called if |V| = 1, |E| = 0)
int CComponent::isolated_vertex(void) const
{
/*
  assert((_V.size() == 1) && (_E.size() == 0));
*/
  return _V._V[0];
}

//----------------------------------------------------------------------------------------------

// Add edge f to the component if possible; return -1 if f is not incident, 0 if f is 
// already present in E and 1 otherwise.
int CComponent::add_edge(const CEdge& f)
{
  if (_V.find_incidences(f) > 0)
    {  
      _V.add(f._v);
      _V.add(f._w);

      return (int)_E.add(f);
    }

  // If edge not incident with component, return -1
  return -1;
}

//----------------------------------------------------------------------------------------------

// Report component
void CComponent::report(void) const
{
  if (_E.size() == 0) 
    {
      cout << "isolated vertex " << isolated_vertex();
    }
  else
    {
      cout << "edges " << _E << " on vertices " << _V;
    }
}

//----------------------------------------------------------------------------------------------
