//*********************************************************************************************
// component.h
//*********************************************************************************************

#ifndef __COMPONENT_H__
#define __COMPONENT_H__

#pragma message("Compiling component.h\n")

// Required header files
#include "edge.h"

// Classes defined in this file
struct less_Component;
class CComponent;

// User-defined types
typedef set<CComponent*, less_Component> Component_set;

//----------------------------------------------------------------------------------------------

// Function object to order components
struct less_Component
{
  bool operator()(const CComponent* c1, const CComponent* c2) const;
};

//----------------------------------------------------------------------------------------------

// Component: connected subgraph (V, E) of graph G = ([n], F), V subset [n], E subset F
class CComponent
{
 private:

  friend class CGraph;
  friend class CForest;
  friend struct less_Component;
  
  // Ordered lists of vertices V and edges E
  CVertices _V;
  CEdges _E;

  CComponent(int v);
  CComponent(const CEdge& e);
  CComponent(const CComponent* c1, const CComponent* c2, const CEdge& e);

  bool operator<(const CComponent& rhs) const;

  int isolated_vertex(void) const;
  int add_edge(const CEdge& f);

  void report(void) const;  
};

//----------------------------------------------------------------------------------------------

#endif
