//*********************************************************************************************
// edge.h
//*********************************************************************************************

#ifndef __EDGE_H__
#define __EDGE_H__

#pragma message("Compiling edge.h\n")

// Externally defined classes
class CPermutation;

// Classes defined in this file
class CEdge;
class CEdges;
class CVertices;

//----------------------------------------------------------------------------------------------

// Undirected edge e = <v, w>
class CEdge
{
 protected:  

  friend class CEdges;
  friend class CVertices;
  friend class CGraph;
  friend class CComponent;

  // End vertices v, w
  int _v;
  int _w;

  bool has_endvertex(int v) const;

 public:

  friend ostream& operator<<(ostream& os, const CEdge& e);

  CEdge() {}
  CEdge(int v, int w);

  bool operator<(const CEdge& rhs) const;
  bool operator==(const CEdge& rhs) const;
};

ostream& operator<<(ostream& os, const CEdge& e);

//----------------------------------------------------------------------------------------------

// List of edges E = (e_1, ..., e_s)
class CEdges
{
 private:  

  friend class CGraph;
  friend class CComponent;

  // List of edges without duplicates
  vector<CEdge> _E;

  CEdges(const CEdge& e) { _E.push_back(e); }
  CEdges(const CEdge& e, const CEdge& f) { _E.push_back(e); add(f); }

  int first_isolated_vertex(void);

  bool operator<(const CEdges& rhs) const;
  bool operator==(const CEdges& rhs) const;

 public:

  friend ostream& operator<<(ostream& os, const CEdges& E);

  CEdges(void) {};
  CEdges(int k, ...);

  bool add(const CEdge& f);
  int add(const CEdges& F);

  bool remove(const CEdge& f);
  int remove(const CEdges& F);

  void push(const CEdge& e) { _E.push_back(e); }
  void pop(void) { _E.pop_back(); }

  void combine(const CEdges& E1, const CEdges& E2);

  int size(void) const { return _E.size(); }
};

ostream& operator<<(ostream& os, const CEdges& E);

//----------------------------------------------------------------------------------------------

// List of vertices V = (v_1, ..., v_n)
class CVertices
{
 private:

  friend class CGraph;
  friend class CComponent;

  vector<int> _V;

  int find_incidences(const CEdge& f) const;

 public:

  friend ostream& operator<<(ostream& os, const CVertices& V);

  CVertices() {}
  CVertices(const vector<int>& V) : _V(V) {}

  bool add(int v);
  bool remove(int v);

  void combine(const CVertices& V1, const CVertices& V2);

  int size(void) const { return _V.size(); }
};

ostream& operator<<(ostream& os, const CVertices& V);

//----------------------------------------------------------------------------------------------

#endif
