//*********************************************************************************************
// forest.cpp
//*********************************************************************************************

#include "StdAfx.h"

#include "forest.h"

//----------------------------------------------------------------------------------------------

// Functor for total ordering on forests
bool less_Forest::operator()(const CForest& F1, const CForest& F2) const
{
  return (F1 < F2);
}

//----------------------------------------------------------------------------------------------

// Construct forest from set of constituent components
CForest::CForest(const Component_set& C)
{
  Component_set::const_iterator c;
  for (c = C.begin(); c != C.end(); c++)
    {
      _C.push_back(*c);
    }
}

//----------------------------------------------------------------------------------------------

// Construct forest from forest F, replacing the two constituent components c1 and c2
// by the new component new_c.
CForest::CForest(const CForest& F, CComponent* c1, CComponent* c2, CComponent* new_c)
{
  bool c1_done = false;
  bool c2_done = false;
  bool new_c_done = false;
  vector<CComponent*>::const_iterator c;

  for (c = F._C.begin(); c != F._C.end(); c++)
    {
      if (*c == c1) 
        {
          c1_done = true;
        }
      else if (*c == c2) 
        {
          c2_done = true;
        }
      else
        {
          if ((!new_c_done) && (*new_c < **c))
            {
              _C.push_back(new_c);
              new_c_done = true;
            }
          _C.push_back(*c);         
        }
    }

/*
  assert(c1_done && c2_done);
*/
  if (!new_c_done) 
    {
      _C.push_back(new_c);
    }
}

//----------------------------------------------------------------------------------------------

// Total order on forests
// Let F, G be forests with ordered lists of components C = (c_1, ..., c_n), D = (d_1, ..., d_m).
// Precedence is given to forests with more components, hence the empty forest is least element.
// F < G :<=> n > m                         if n != m
//            C < D under component order   otherwise
bool CForest::operator <(const CForest& rhs) const
{
  int nc = _C.size();
  int rhsnc = rhs._C.size();

  if (nc != rhsnc) return (nc > rhsnc);
  return lexicographical_compare(_C.begin(), _C.end(), rhs._C.begin(), rhs._C.end());
}

//----------------------------------------------------------------------------------------------

// Report forest
void CForest::report(void) const
{
  cout << "forest with components: " << endl;

  int nc = _C.size();
  for (int i=0; i<nc; i++)
    {
      cout << "- component " << i+1<< ": ";
      _C[i]->report();
      cout << endl;
    }
}

//----------------------------------------------------------------------------------------------
