//*********************************************************************************************
// graphutils.h
//*********************************************************************************************

#ifndef __GRAPHUTILS_H__
#define __GRAPHUTILS_H__

#pragma message("Compiling graphutils.h\n")

// Required header files
#define EXTDEFS 1
#include "Nauty20/nauty.h"

#include "edge.h"

//----------------------------------------------------------------------------------------------

// Classes defined in this file
class CG_entry;
class CM_value;

// User-defined types
typedef vector< setword > Graph_code;
typedef vector< CG_entry > Graph_list;
typedef pair< Graph_code, Graph_code > M_index;
typedef map< M_index, CM_value > M_map;
typedef vector< M_map > M_maps;

//----------------------------------------------------------------------------------------------

// Data structure for entries of graph list
class CG_entry
{
 private:
  friend class CGraph;

  // Graph G giving rise to a new instance (G, e_i, f_i), for some (or both) i=1,2.
  Graph_code _code;

  // Subscript i for which the instance (G, e_i, f_i) was found equivalent to some earlier 
  // instance (or -1 if both (G, e_1, f_1) and (G, e_2, f_2) gave rise to new instances)
  int _skip;

  CG_entry(Graph_code& code, int skip) : _code(code), _skip(skip) {};
};

//----------------------------------------------------------------------------------------------

// Data structure for values M^X(D)
class CM_value
{
 private:

  friend class CGraph;

  // Value M^X(D)
  long _M;

  // Origin of the value
  // 1 = M_1 obtained in CGraph::check_instance
  // 2 = M_tot obtained in CGraph::compute_M while summing over M_minors
  // 3 = M_minor computed in CGraph::obtain_M_minor using G.count_subforests(U, V)
  int _origin;

  CM_value(long M, int origin) : _M(M), _origin(origin) {}
};

//----------------------------------------------------------------------------------------------

#endif