//*********************************************************************************************
// permutation.cpp
//*********************************************************************************************

#include "StdAfx.h"

#include "permutation.h"

//----------------------------------------------------------------------------------------------

// Construct identity permutation id : x |-> x on [n]
CPermutation::CPermutation(int n)
{
  _image.resize(n);
  _DCN_available = false;
  
  for(int x=0; x<n; x++)
    {
      _image[x] = x+1;
    }
}

//----------------------------------------------------------------------------------------------

// Compute gamma(x) for x in [n]
int CPermutation::permute(int x) const
{ 
/*
  assert((x >= 1) && (x <= _image.size()));
*/
  return _image[x-1]; 
}

//----------------------------------------------------------------------------------------------

// Compute inverse gamma^{-1}
void CPermutation::inverse(CPermutation& inv) const
{
  int n = _image.size();
  inv._image.resize(n);
  
  for (int x=0; x<n; x++)
    {
      inv._image[_image[x]-1] = x+1;
    }
}

//----------------------------------------------------------------------------------------------

// Check for identity permutation id : x |-> x (which has trivial DCN)
bool CPermutation::is_id(void) const
{
  int n = _image.size();
  for (int x=1; x<=n; x++)
    {
      if (permute(x) != x) return false;
    }

  return true;
}

//----------------------------------------------------------------------------------------------

// Computes disjoint cycle notation
void CPermutation::compute_DCN(void)
{
  if (_DCN_available) return;

  int n = _image.size();
  vector<int> cycle;
  vector<bool> visited(n+1, false);
  int x, i;
	
  _DCN.resize(0);
  x = 1;

  while (x <= n)
    {
      cycle.resize(0);
      i = x;

      // Compute next cycle
      do 
        {
          cycle.push_back(i);
          visited[i-1] = true;
          i = permute(i);
        } 
      while (i != x);
	
      // Store cycle unless trival
      if (cycle.size() > 1) _DCN.push_back(cycle);

      // Find next unvisited x in X
      while (visited[++x-1]) {};
    }

  _DCN_available = true;
}

//----------------------------------------------------------------------------------------------

// Reports permutation gamma in disjoint cycle notation
void CPermutation::report(void)
{
/*
  assert(_image.size() > 0);
*/

  if (is_id())
    {
      cout << "id";
    }
  else
    {
      compute_DCN();

      int DCN_size = _DCN.size();
      for (int i = 0; i < DCN_size; i++)
        {
          cout << _DCN[i];
        }
    }
}

//----------------------------------------------------------------------------------------------
