//*********************************************************************************************
// ratiostats.cpp
//*********************************************************************************************

#include "StdAfx.h"

#include "graph.h"

#include "ratiostats.h"

//----------------------------------------------------------------------------------------------

CRatiostats::CRatiostats(int n, int N, double res, double range_min, int E_i)
{ 
  _n = n;
  _N = N;

  _num_instances.resize(N, 0);

  _mean.resize(N, 0.0); 
  _min.resize(N, MAXDOUBLE); 
  _max.resize(N, 0.0);

  _finalized.resize(N, false);

  _res = res;
  _range_min = range_min;
  _num_buckets.resize(N, 1);
  _histogram.resize(N, vector<int>(1, 0));

  string subtitle = string("Buffer.n_") + _n;
  if (E_i >= 0) subtitle = subtitle + string(".E_") + E_i;

//  _buffer1 = new ofstream((PATH + subtitle + ".1.txt").c_str());
//  _buffer2 = new ofstream((PATH + subtitle + ".2.txt").c_str());

//  *_buffer1 << "result := [";
}

//----------------------------------------------------------------------------------------------

CRatiostats::~CRatiostats(void) 
{
//  *_buffer1 << "];";

 // _buffer1->close(); 
 // _buffer2->close(); 
  
//  delete _buffer1; 
//  delete _buffer2; 
}

//----------------------------------------------------------------------------------------------

void CRatiostats::update(int s, double ratio, CGraph& G)
{
/*
  assert((s >= 0) && (s < _N));
*/
  _num_instances[s]++;

  _mean[s] += ratio;
  if (ratio < _min[s]) 
    {
      _min[s] = ratio;
      _G_min.clear();
    }
  if (ratio > _max[s]) 
    {
      _max[s] = ratio;
      _G_max.clear();
    }

  if (ratio == _min[s]) _G_min.push_back(G);
  if (ratio == _max[s]) _G_max.push_back(G);

  int bucket = floor((ratio - _range_min)/_res);
  if (bucket >= _num_buckets[s])
    {
      _histogram[s].resize(bucket+1, 0);
      _num_buckets[s] = bucket+1;
    }

  _histogram[s][bucket]++;
}

//----------------------------------------------------------------------------------------------

void CRatiostats::finalize(int s)
{
/*
  assert(!_finalized[s]);
*/
  _finalized[s] = true;

  _mean[s] /= (double)_num_instances[s];
/*
  int i, num_G;
  *_buffer1 << "[[" << s << ", " << _mean[s] << ", " << _min[s] << ", " << _max[s] << "], ["; 
  num_G = _G_min.size();
  for (i = 0; i < num_G; i++) 
    {
      *_buffer1 << _G_min[i];
      if (i < num_G-1)  *_buffer1 << ", ";
    }
  *_buffer1 << "], [";
  num_G = _G_max.size();
  for (i = 0; i < num_G; i++)
    {
      *_buffer1 << _G_max[i];
      if (i < num_G-1)  *_buffer1 << ", ";
    }
  *_buffer1 << "]]";
  if (s < _N-1) *_buffer1 << ", ";
  *_buffer1 << endl;
  _buffer1->flush();

  *_buffer2 << s << "\t";
  for (int c = 0; c < _num_buckets[s]; c++)
    {
      *_buffer2 << _histogram[s][c] << "\t";
    }
  *_buffer2 << endl;
  _buffer2->flush();
*/
}

//----------------------------------------------------------------------------------------------

void CRatiostats::report(int s)
{
/*
  assert((s >= 0) && (s < _N));
*/
  if (!_finalized[s]) finalize(s);

  cout << "Average ratio " << _mean[s] << endl;
  cout << "Minimum ratio " << _min[s] << endl;
  cout << "Maximum ratio " << _max[s];
}

//----------------------------------------------------------------------------------------------

void CRatiostats::plot(int E_i)
{
  int s;

  // Plot average, min and max ratios vs layer
  vector<int> X(_N-3);
  for (s = 3; s < _N; s++) X[s-3] = s;

  string subtitle = string("n_") + _n;
  if (E_i >= 0) subtitle = subtitle + string(".E_") + E_i;

  {
    matrix<double> Z(_N-3, vector<double>(3, 0.0));
    for (s = 3; s < _N; s++) 
      {
        if (!_finalized[s]) finalize(s);

        Z[s-3][0] = _mean[s];
        Z[s-3][1] = _min[s];
        Z[s-3][2] = _max[s];
      }

    vector<string> label(3);
    label[0] = "average";
    label[1] = "min";
    label[2] = "max";

    vector<const char*> Y(3);
    for (int i = 0; i < 3; i++) Y[i] = label[i].c_str();

    string title = string("s vs ratio (") + subtitle + ")";

    string filename = PATH + "Ratio.plot." + subtitle + ".txt";

    plotXYZ(X, Y, Z, title.c_str(), filename);
  }

  // Plot ratio histograms for each layer
  {
    int C = max(_num_buckets), c;

    matrix<int> Z(_N-3, vector<int>(C, 0));
    for (s = 3; s < _N; s++) 
      {
        for (c = 0; c < _num_buckets[s]; c++)
          {
            Z[s-3][c] = _histogram[s][c];
          }
      }

    vector<double> Y(C);
    for (c = 0; c < C; c++)
      {
        Y[c] = (double)c*_res + _range_min;
      }
    
    string title = string("s vs # G with ratio in [r, r+dr] (") + subtitle + ")";

    string filename = PATH + "Ratio.histogram." + subtitle + ".txt";

    plotXYZ(X, Y, Z, title.c_str(), filename);
  }
}

//----------------------------------------------------------------------------------------------
